/*
 * Copyright (c) 2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 ** morTrackTaskCodels.c
 **
 ** Codels called by execution task morTrackTask
 **
 **/

#include <math.h>
#include <portLib.h>
#include <h2mathLib.h>

#include "server/morHeader.h"
#include "morConst.h"

static GENPOS_CART_REF stillRef;

/*------------------------------------------------------------------------
 * Track
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mor_POSTER_NOT_FOUND
 *              S_mor_BAD_REF
 *              S_mor_CMD_STOP_TRACK
 */

/* findTrackedPoster  -  codel START of Track
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
findTrackedPoster(MOR_TRACK_PARAMS *params, int *report)
{
	char *trackedPoster = (char *)params->posterName.name;
	POSTER_ID posterTrackedId;
	GENPOS_CART_STATE *robot = &SDI_F->robot;

	/* Poster Identifier */
	if (posterFind((char *) trackedPoster, &posterTrackedId) != OK)  {
		*report = S_mor_POSTER_NOT_FOUND;
		return(END);
	}
	morDataStrId->trackStr.posterId = (int) posterTrackedId;
	morDataStrId->mode = MOR_TRACK;
	/* Set stillRef to current position of the robot */
	stillRef.x = robot->xRef;
	stillRef.y = robot->yRef;
	stillRef.theta = robot->theta;
	stillRef.v = 0.0;
	stillRef.w = 0.0;
	stillRef.vmax = MOR_VMAX;
	stillRef.wmax = MOR_WMAX;
	stillRef.linAccelMax = MOR_ACCEL_LIN_MAX;
	stillRef.angAccelMax = MOR_ACCEL_ANG_MAX;
	switch (params->mode) {
	case GENPOS_TRACK_CONFIG:
		morDataStrId->ref.dataType = GENPOS_POS_AND_SPEED_DATA;
		stillRef.dataType = GENPOS_POS_AND_SPEED_DATA;
		break;
	case GENPOS_TRACK_SPEED:
		morDataStrId->ref.dataType = GENPOS_SPEED_DATA;
		stillRef.dataType = GENPOS_SPEED_DATA;
		break;
	default:		
		*report = S_mor_BAD_REF;
		return FAIL;
	}
	stillRef.numRef = -1;

	printf("mor tracking on %s\n", (char *)trackedPoster);
	return(EXEC);
}

static ACTIVITY_EVENT
pumpConfigReference(int *report)
{
	GENPOS_CART_REF morOrders;
	GENPOS_CART_REF *refId = &morDataStrId->ref;
	GENPOS_CART_STATE *configId = &morDataStrId->robot;
	static int prevRef = -2;

	/* Lire les consignes sur le poster externe */
	if (posterRead ((POSTER_ID)morDataStrId->trackStr.posterId, 0,
		(void *) &morOrders,
		sizeof (GENPOS_CART_REF)) == sizeof (GENPOS_CART_REF)) {

		/* Check validity of that  */
		if (morOrders.numRef == -1) {
			if (prevRef != -1) {
				printf("Pilo numRef = -1\n");
			}
			prevRef = -1;
			/* Not a valid reference don't move */
			memcpy(refId, &stillRef, sizeof(GENPOS_CART_REF));
			return (EXEC);
		}
		/* Limit speeds and accelerations */
		/* XXX may be set an error */
		if (morOrders.vmax > MOR_VMAX)
			morOrders.vmax = MOR_VMAX;
		if (morOrders.wmax > MOR_WMAX)
			morOrders.wmax = MOR_WMAX;
		if (morOrders.linAccelMax > MOR_ACCEL_LIN_MAX)
			morOrders.linAccelMax = MOR_ACCEL_LIN_MAX;
		if (morOrders.angAccelMax > MOR_ACCEL_ANG_MAX)
			morOrders.angAccelMax = MOR_ACCEL_ANG_MAX;

		/* Send reference */
		refId->backFlag = (morOrders.v > 0 ?
				   GENPOS_FORWARD_MOTION :
				   GENPOS_BACKWARD_MOTION);
		refId->v = fabs(morOrders.v);
		refId->vt = 0;
		refId->w = morOrders.w;
		refId->vmax = morOrders.vmax;
		refId->wmax = morOrders.wmax;
		refId->linAccelMax = morOrders.linAccelMax;
		refId->angAccelMax = morOrders.angAccelMax;
		refId->x = morOrders.x;
		refId->y = morOrders.y;
		refId->theta =  angleLimit(morOrders.theta);
		refId->dataType = GENPOS_POS_AND_SPEED_DATA;
		/* store position in case of stop */
		stillRef.x = refId->x;
		stillRef.y = refId->y;
		stillRef.theta = refId->theta;
		prevRef = morOrders.numRef;
		return(EXEC);
	}
	*report = errnoGet();
	return(END);
}

static ACTIVITY_EVENT
pumpSpeedReference(int *report)
{
	GENPOS_CART_SPEED morOrders;
	GENPOS_CART_REF *refId = &morDataStrId->ref;
	GENPOS_CART_STATE *configId = &morDataStrId->robot;

	/* Lire les consignes sur le poster externe */
	if (posterRead ((POSTER_ID)morDataStrId->trackStr.posterId, 0,
		(void *) &morOrders,
		sizeof (GENPOS_CART_SPEED)) == sizeof (GENPOS_CART_SPEED)) {

		/* Verifier la validite des arguments */
		if (morOrders.v > MOR_VMAX ||
		    morOrders.w > MOR_WMAX) {

			/* On met une consigne de vitesse nulle sur le point
			 courant. XXXX ouch. */
			refId->v = 0.;
			refId->w = 0.;
			refId->dataType = GENPOS_SPEED_DATA;
			*report = S_mor_BAD_REF;
			return (END);
		}

		/* Transmettre la consigne */
		else {
			refId->backFlag = (morOrders.v > 0 ?
			    GENPOS_FORWARD_MOTION : GENPOS_BACKWARD_MOTION);
			refId->v = fabs(morOrders.v);
			/* printf("pumpSpeedRef: %lf\n", refId->v); */
			refId->vt = 0;
			refId->w = morOrders.w;
			refId->vmax = morOrders.vmax;
			refId->wmax = morOrders.wmax;
			refId->linAccelMax = morOrders.linAccelMax;
			refId->angAccelMax = morOrders.angAccelMax;
			refId->dataType = GENPOS_SPEED_DATA;
			return(EXEC);
		}
	} else {
		printf("pumpSpeedReference: posterRead error\n");
	}
	*report = errnoGet();
	return(END);
}
/* pumpReference  -  codel EXEC of Track
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
pumpReference(MOR_TRACK_PARAMS *params, int *report)
{
	GENPOS_TRACK *track = &morDataStrId->trackStr;

	/* Check if mode changed */
	switch (SDI_F->mode) {
	case MOR_EMERGENCY:
		*report = S_mor_EMERGENCY_STOP;
		return END;
	case MOR_MOTORS_OFF:
		*report = S_mor_MOTORS_OFF;
		return END;
	}

	switch (track->trackMode) {
	case GENPOS_TRACK_POS:
		return pumpConfigReference(report);
	case GENPOS_TRACK_CONFIG:
		return pumpConfigReference(report);
	case GENPOS_TRACK_SPEED:
		return pumpSpeedReference(report);
	default:
		*report = EXEC_TASK_BILAN(MOR_MOTIONTASK_NUM);
		return(END);

	}
}

/* smoothStopTrack  -  codel END of Track
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
smoothStopTrack(MOR_TRACK_PARAMS *params, int *report)
{
	GENPOS_CART_REF *refId;	       /* Id structure de consignes */
	GENPOS_CART_STATE *robotId;	/* Id config du robot */
	GENPOS_TRACK *trackId;
	double dtArret;                /* Temps d'arret */
	double dtRetard;               /* Temps de prise en compte de la consigne */
	double dsArret;                /* Longueur d'arret */
	double dtLoco;                 /* Periode d'asserv de la loco */
	double angle;                  /* Moyenne entre angles d'init et d'arret */
	double theta;

	printf("mor smoothStopTrack\n");

	/* Obtenir les pointeurs vers les structures de donnees de la locomotion */
	robotId = &(morDataStrId->robot);
	refId = &morDataStrId->ref;
	trackId = &morDataStrId->trackStr;

	if (trackId->trackMode == GENPOS_TRACK_SPEED) {
		/* Set a null speed and stop the tracking */
		trackId->trackMode = GENPOS_NO_TRACKING;
		refId->dataType = GENPOS_SPEED_DATA;
		refId->x = robotId->xRob;
		refId->y = robotId->yRob;
		refId->theta = robotId->theta;
		refId->v = 0.0;
		refId->w = 0.0;
		return ETHER;
	}

	/* Determination de la periode d'asserv de la loco */
	dtLoco = EXEC_TASK_PERIOD(MOR_TRACKTASK_NUM);

	/* Temps d'arret minimal (tps de decel) */
	dtArret = MAX (fabs(robotId->v) / MOR_ACCEL_LIN_MAX,
	    fabs(robotId->w) / MOR_ACCEL_ANG_MAX);

	/* Temps de prise en compte (v = cst) */
	dtRetard =  1 * dtLoco;

	/* Angle final (avec periode de prise en compte) */
	theta = robotId->theta + robotId->w * (dtArret / 2 + dtRetard);

	/* Angle moyen */
	angle = (robotId->theta + theta) / 2;

	/* Longueur d'arret */
	dsArret = robotId->v * (dtArret / 2 + dtRetard);

	/* Prevision position du centre */
	refId->x = robotId->xRob + dsArret * cos (angle);
	refId->y = robotId->yRob + dsArret * sin (angle);

	/* Vitesses de consigne nulles */
	refId->v = 0. ;
	refId->w = 0. ;

	/* Indiquer l'arret du tracking */
	refId->dataType = GENPOS_POS_AND_SPEED_DATA;
	trackId->trackMode = GENPOS_NO_TRACKING;
	morDataStrId->mode = MOR_READY;
	return ETHER;
}

/* smoothStopTrack  -  codel INTER of Track
   see above (request Track) */

