/*
 * Copyright (c) 2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _MORCONST_H
#define _MORCONST_H

#define MOR_DIST_TO_CONTROLLED_POINT 0.0

/* Default uncertainties */
#define MOR_VAR_XX	0.001
#define MOR_VAR_YY	0.001
#define MOR_VAR_TT	0.0001
#define MOR_VAR_QCOEFF	0.05

/* Maximum speed */
#define MOR_VMAX	0.95	/* 1m/s */
#define MOR_WMAX	3.1416

/* Maximum accelerations */
#define MOR_ACCEL_LIN_MAX	5.0
#define MOR_ACCEL_ANG_MAX	(3*M_PI/2)

/* Control Params */
#define MOR_KP_LONGIT	2.
#define MOR_KI_LONGIT	1.

#define MOR_KP_TRANSV	10.
#define MOR_KI_TRANSV	6.

#define MOR_KP_ANG		2.
#define MOR_KI_ANG		2.

/* max errors */
#define MOR_LONGIT_MAX_GAP	0.5
#define MOR_TRANSV_MAX_GAP	0.5
#define MOR_ANG_MAX_GAP	0.5

#define MOR_LONGIT_ERROR_MAX	0.5
#define MOR_TRANSV_ERROR_MAX	0.5
#define MOR_ANG_ERROR_MAX	0.5

#define MOR_LONGIT_MAX_ERR_SUM	1.0
#define MOR_TRANSV_MAX_ERR_SUM	1.0
#define MOR_ANG_MAX_ERR_SUM		1.0

/* spine */

#define MOR_SPINE_POS_MIN 0.00
#define MOR_SPINE_POS_MAX 0.33
#define MOR_SPINE_SPEED_MIN 0.005
#define MOR_SPINE_SPEED_MAX 0.15

/* joystick */
#define MOR_JOYSTICK_DEVICE_PATH "/dev/input/js0"
#define MOR_JOYSTICK_VGAIN	0.5/(32768.0*32768.0) /* 1.2 m/s max */
#define MOR_JOYSTICK_WGAIN   0.7/(32768.0*32768.0) /* 0.7 rad/s max */

#endif
